@@ -13,6 +13,17 @@ public class CmdFilterSuggestions implements Command {
 	public int getIconFuzzy() { return R.drawable.ic_fn_filter_fuzzy; }
 	public int getName() { return R.string.function_filter_suggestions; }
 
+	@Override
+	public boolean isAvailable(@Nullable TraditionalT9 tt9) {
+		return
+			tt9 != null
+			&& tt9.isFilteringSupported()
+			&& !tt9.isInputModeABC()
+			&& !tt9.isInputModeNumeric()
+			&& !tt9.isVoiceInputActive()
+			&& !tt9.isFnPanelVisible();
+	}
+
 	public boolean run(@Nullable TraditionalT9 tt9, boolean repeat) {
 		return tt9 != null && tt9.onKeyFilterSuggestions(false, repeat);
 	}
