@@ -12,7 +12,6 @@ import java.util.ArrayList;
 
 import io.github.sspanak.tt9.R;
 import io.github.sspanak.tt9.db.words.DictionaryLoader;
-import io.github.sspanak.tt9.hacks.AppHacks;
 import io.github.sspanak.tt9.hacks.InputType;
 import io.github.sspanak.tt9.ime.helpers.CursorOps;
 import io.github.sspanak.tt9.ime.helpers.InputConnectionAsync;
@@ -32,14 +31,13 @@ import io.github.sspanak.tt9.util.chars.Characters;
 
 public abstract class TypingHandler extends KeyPadHandler {
 	// internal settings/data
-	@NonNull protected AppHacks appHacks = new AppHacks(null, null, null);
 	@NonNull protected InputType inputType = new InputType(null, null);
 	@NonNull protected TextField textField = new TextField(null, null, null);
 	@NonNull protected TextSelection textSelection = new TextSelection(null, null);
 	@NonNull protected SuggestionOps suggestionOps = new SuggestionOps(null, null, null, null, null, null, null, null);
 
-	@Nullable private Handler shiftStateDebounceHandler;
 	@Nullable private Handler suggestionHandler;
+	@Nullable private Handler shiftStateDebounceHandler;
 
 	// input
 	@NonNull protected ArrayList<Integer> allowedInputModes = new ArrayList<>();
@@ -59,9 +57,10 @@ public abstract class TypingHandler extends KeyPadHandler {
 		return getCurrentInputConnection() == null || InputModeKind.isPassthrough(mInputMode);
 	}
 
+
 	@Override
-	protected boolean onStart(EditorInfo field) {
-		boolean restart = textField.equals(getCurrentInputConnection(), field);
+	protected boolean onStart(EditorInfo field, boolean restarting) {
+		boolean restart = restarting || textField.equals(getCurrentInputConnection(), field);
 
 		setInputField(field);
 
@@ -79,7 +78,7 @@ public abstract class TypingHandler extends KeyPadHandler {
 		resetKeyRepeat();
 		mInputMode = determineInputMode();
 		determineTextCase();
-		updateShiftState(true, false);
+		updateShiftState(null, true, false); // don't use beforeCursor cache on start up
 		suggestionOps.set(null);
 
 		return true;
@@ -104,7 +103,7 @@ public abstract class TypingHandler extends KeyPadHandler {
 		textSelection = new TextSelection(context, inputType);
 
 		// changing the TextField and notifying all interested classes is an atomic operation
-		appHacks = new AppHacks(inputType, textField, textSelection);
+		appHacks.setDependencies(inputType, textField, textSelection);
 		suggestionOps.setDependencies(appHacks, textField, statusBar);
 	}
 
@@ -162,7 +161,7 @@ public abstract class TypingHandler extends KeyPadHandler {
 			suggestionOps.commitCurrent(false, true);
 			mInputMode.reset();
 			deleteText(settings.getBackspaceAcceleration() && repeat > 0);
-			updateShiftStateDebounced(mInputMode.getSuggestions().isEmpty(), false);
+			updateShiftStateDebounced(null, mInputMode.noSuggestions(), false); // backspace may change the text too much, so no beforeCursor cache for now
 			recompose(repeat, !textSelection.isEmpty());
 		}
 
@@ -182,36 +181,30 @@ public abstract class TypingHandler extends KeyPadHandler {
 		suggestionOps.cancelDelayedAccept();
 
 		hold = hold && settings.getHoldToType();
+		String[] surroundingChars = textField.getSurroundingStringForAutoAssistance(settings, mInputMode);
 
-		// In Korean, the next char may "steal" components from the previous one, in which case,
-		// we must replace the previous char with a one containing less strokes.
-		if (mInputMode.shouldReplaceLastLetter(key, hold)) {
-			mInputMode.replaceLastLetter();
-		}
 		// Automatically accept the previous word, when the next one is a space or punctuation,
 		// instead of requiring "OK" before that.
 		// First pass, analyze the incoming key press and decide whether it could be the start of
 		// a new word. In case we do accept it, we preserve the suggestion list instead of clearing,
 		// to prevent flashing while the next suggestions are being loaded.
-		else if (mInputMode.shouldAcceptPreviousSuggestion(suggestionOps.getCurrent(), key, hold)) {
+		if (mInputMode.shouldAcceptPreviousSuggestion(suggestionOps.getCurrent(), key, hold)) {
 			// WARNING! Ensure the code after "acceptIncompleteAndKeepList()" does not depend on
 			// the suggestions in SuggestionOps, since we don't clear that list.
 			String lastWord = suggestionOps.acceptIncompleteAndKeepList();
 			mInputMode.onAcceptSuggestion(lastWord);
-			autoCorrectSpace(lastWord, false, key);
+			surroundingChars = autoCorrectSpace(lastWord, surroundingChars, false, key);
 		}
 
-		// Auto-adjust the text case before each word, if the InputMode supports it.
-		if (mInputMode.getSuggestions().isEmpty()) {
-			mInputMode.determineNextWordTextCase(key);
-		}
+		// Auto-adjust the text case before each word/char, if the InputMode supports it.
+		mInputMode.determineNextWordTextCase(surroundingChars[0], key);
 
-		if (!mInputMode.onNumber(key, hold, repeat)) {
+		if (!mInputMode.onNumber(key, hold, repeat, surroundingChars)) {
 			forceShowWindow();
 			return false;
 		}
 
-		if (mInputMode.shouldSelectNextSuggestion() && !mInputMode.getSuggestions().isEmpty()) {
+		if (mInputMode.shouldSelectNextSuggestion() && !mInputMode.noSuggestions()) {
 			scrollSuggestions(false);
 			suggestionOps.scheduleDelayedAccept(mInputMode.getAutoAcceptTimeout());
 		} else {
@@ -233,37 +226,66 @@ public abstract class TypingHandler extends KeyPadHandler {
 
 		suggestionOps.cancelDelayedAccept();
 
+		String[] surroundingChars;
+
 		// accept the previously typed word (if any)
 		String lastWord = suggestionOps.acceptIncomplete();
-		if (!lastWord.isEmpty()) {
+		if (lastWord.isEmpty()) {
+			surroundingChars = textField.getSurroundingStringForAutoAssistance(settings, mInputMode);
+		} else {
 			mInputMode.onAcceptSuggestion(lastWord);
-			autoCorrectSpace(lastWord, false, -1);
+			surroundingChars = autoCorrectSpace(
+				lastWord,
+				textField.getSurroundingStringForAutoAssistance(settings, mInputMode),
+				false,
+				-1
+			);
 		}
 
 		// "type" and accept the new word
 		mInputMode.onAcceptSuggestion(text);
 		textField.setText(text);
-		autoCorrectSpace(text, true, -1);
+		surroundingChars[0] += text;
+		String beforeCursor = autoCorrectSpace(text, surroundingChars, true, -1)[0];
 
 		forceShowWindow();
-		updateShiftState(true, false);
+		updateShiftState(beforeCursor, true, false);
 
 		return true;
 	}
 
 
-	private void autoCorrectSpace(String currentWord, boolean isWordAcceptedManually, int nextKey) {
-		if (!inputType.isRustDesk() && mInputMode.shouldDeletePrecedingSpace()) {
+	@NonNull
+	private String[] autoCorrectSpace(@Nullable String currentWord, @NonNull String[] surroundingChars, boolean isWordAcceptedManually, int nextKey) {
+		if (currentWord == null || currentWord.isEmpty() || !settings.isAutoAssistanceOn(mInputMode)) {
+			return surroundingChars;
+		}
+
+		String previousChars = surroundingChars[0];
+		final String nextChars = surroundingChars[1];
+
+		if (!inputType.isRustDesk() && mInputMode.shouldDeletePrecedingSpace(previousChars)) {
 			textField.deletePrecedingSpace(currentWord);
+			if (previousChars.endsWith(" " + currentWord) && previousChars.length() > currentWord.length()) {
+				final int precedingSpace = previousChars.length() - currentWord.length() - 1;
+				previousChars = previousChars.substring(0, precedingSpace) + currentWord;
+			}
 		}
 
-		if (mInputMode.shouldAddPrecedingSpace()) {
+		if (mInputMode.shouldAddPrecedingSpace(previousChars)) {
 			textField.addPrecedingSpace(currentWord);
+			if (previousChars.endsWith(currentWord)) {
+				final int startOfWord = previousChars.length() - currentWord.length();
+				previousChars = previousChars.substring(0, startOfWord) + " " + currentWord;
+			}
 		}
 
-		if (mInputMode.shouldAddTrailingSpace(isWordAcceptedManually, nextKey)) {
+		if (mInputMode.shouldAddTrailingSpace(previousChars, nextChars, isWordAcceptedManually, nextKey)) {
 			textField.setText(" ");
+			previousChars += " ";
 		}
+
+		return new String[] { previousChars, nextChars };
 	}
 
 
@@ -409,22 +431,43 @@ public abstract class TypingHandler extends KeyPadHandler {
 	}
 
 
-	protected void onAcceptSuggestionAutomatically(String word) {
-		mInputMode.onAcceptSuggestion(word, true);
-		autoCorrectSpace(word, false, mInputMode.getFirstKey());
-		mInputMode.determineNextWordTextCase(-1);
+	private String onAcceptPreviousSuggestion() {
+		final int lastWordLength = InputModeKind.isABC(mInputMode) ? 1 : mInputMode.getSequenceLength() - 1;
+		String lastWord = suggestionOps.getCurrent(mLanguage, lastWordLength);
+		if (Characters.PLACEHOLDER.equals(lastWord)) {
+			lastWord = "";
+		}
+
+		suggestionOps.commitCurrent(false, true);
+		mInputMode.onAcceptSuggestion(lastWord, true);
+		final String beforeCursor = autoCorrectSpace(
+			lastWord,
+			textField.getSurroundingStringForAutoAssistance(settings, mInputMode),
+			false,
+			mInputMode.getFirstKey()
+		)[0];
+		mInputMode.determineNextWordTextCase(beforeCursor, -1);
+
+		return beforeCursor;
 	}
 
+
 	private void onAcceptSuggestionsDelayed(String word) {
 		onAcceptSuggestionManually(word, -1);
 		forceShowWindow();
 	}
 
+
 	protected void onAcceptSuggestionManually(String word, int fromKey) {
 		mInputMode.onAcceptSuggestion(word);
 		if (!word.isEmpty()) {
-			autoCorrectSpace(word, true, fromKey);
-			updateShiftState(true, false);
+			String beforeCursor = autoCorrectSpace(
+				word,
+				textField.getSurroundingStringForAutoAssistance(settings, mInputMode),
+				true,
+				fromKey
+			)[0];
+			updateShiftState(beforeCursor, true, false);
 			resetKeyRepeat();
 		}
 
@@ -484,29 +527,31 @@ public abstract class TypingHandler extends KeyPadHandler {
 		// Second pass, analyze the available suggestions and decide if combining them with the
 		// last key press makes up a compound word like: (it)'s, (I)'ve, l'(oiseau), or it is
 		// just the end of a sentence, like: "word." or "another?"
+		String beforeCursor = null;
 		if (mInputMode.shouldAcceptPreviousSuggestion(suggestionOps.getCurrent())) {
-			String lastWord = suggestionOps.acceptPrevious(mLanguage, mInputMode.getSequenceLength());
-			onAcceptSuggestionAutomatically(lastWord);
+			beforeCursor = onAcceptPreviousSuggestion();
 		}
 
-		// display the word suggestions
-		suggestionOps.set(mInputMode.getSuggestions(), mInputMode.containsGeneratedSuggestions());
+		final ArrayList<String> suggestions = mInputMode.getSuggestions();
+		suggestionOps.set(suggestions, mInputMode.containsGeneratedSuggestions());
 
-		// flush the first suggestion, if the InputMode has requested it
+		// either accept the first one automatically (when switching from punctuation to text
+		// or vice versa), or schedule auto-accept in N seconds (in ABC mode)
 		if (suggestionOps.scheduleDelayedAccept(mInputMode.getAutoAcceptTimeout())) {
 			return;
 		}
 
-		// Otherwise, put the first suggestion in the text field,
-		// but cut it off to the length of the sequence (how many keys were pressed),
-		// for a more intuitive experience.
+		// We have not accepted anything yet, which means the user is composing a word.
+		// put the first suggestion in the text field, but cut it off to the length of the sequence
+		// (the count of key presses), for a more intuitive experience.
 		String trimmedWord = suggestionOps.getCurrent(mLanguage, mInputMode.getSequenceLength());
-		appHacks.setComposingTextWithHighlightedStem(trimmedWord, mInputMode);
+		appHacks.setComposingTextWithHighlightedStem(trimmedWord, mInputMode.getWordStem(), mInputMode.isStemFilterFuzzy());
 
-		if (mInputMode.getSuggestions().isEmpty()) {
-			updateShiftStateDebounced(true, false);
+		beforeCursor = beforeCursor != null ? beforeCursor + trimmedWord : trimmedWord;
+		if (suggestions.isEmpty()) {
+			updateShiftStateDebounced(beforeCursor, true, false);
 		} else {
-			updateShiftStateDebounced(false, true);
+			updateShiftStateDebounced(beforeCursor, false, true);
 		}
 
 		forceShowWindow();
@@ -517,27 +562,28 @@ public abstract class TypingHandler extends KeyPadHandler {
 		suggestionOps.cancelDelayedAccept();
 		suggestionOps.scrollTo(backward ? -1 : 1);
 		mInputMode.setWordStem(suggestionOps.getCurrent(), true);
-		appHacks.setComposingTextWithHighlightedStem(suggestionOps.getCurrent(), mInputMode);
+		appHacks.setComposingTextWithHighlightedStem(suggestionOps.getCurrent(), mInputMode.getWordStem(), mInputMode.isStemFilterFuzzy());
 	}
 
 
-	protected void updateShiftStateDebounced(boolean determineTextCase, boolean onlyWhenLetters) {
+	protected void updateShiftStateDebounced(@Nullable String beforeCursor, boolean determineTextCase, boolean onlyWhenLetters) {
 		if (shiftStateDebounceHandler == null) {
 			shiftStateDebounceHandler = new Handler(Looper.getMainLooper());
 		} else {
 			shiftStateDebounceHandler.removeCallbacksAndMessages(null);
 		}
-		shiftStateDebounceHandler.postDelayed(() -> updateShiftState(determineTextCase, onlyWhenLetters), SettingsStore.SHIFT_STATE_DEBOUNCE_TIME);
+		shiftStateDebounceHandler.postDelayed(() -> updateShiftState(beforeCursor, determineTextCase, onlyWhenLetters), SettingsStore.SHIFT_STATE_DEBOUNCE_TIME);
 	}
 
 
-	protected void updateShiftState(boolean determineTextCase, boolean onlyWhenLetters) {
+	protected void updateShiftState(@Nullable String beforeCursor, boolean determineTextCase, boolean onlyWhenLetters) {
 		if (onlyWhenLetters && !new Text(suggestionOps.getCurrent()).isAlphabetic()) {
 			return;
 		}
 
 		if (determineTextCase) {
-			mInputMode.determineNextWordTextCase(-1);
+			beforeCursor = beforeCursor != null ? beforeCursor : textField.getStringBeforeCursor();
+			mInputMode.determineNextWordTextCase(beforeCursor, -1);
 		}
 
 		getDisplayTextCase(mLanguage, mInputMode.getTextCase());
