@@ -23,6 +23,7 @@ import io.github.sspanak.tt9.util.chars.Characters;
 class ModeWords extends ModeCheonjiin {
 	private final String LOG_TAG = getClass().getSimpleName();
 
+	private boolean ignoreNextSpace = false;
 	private String lastAcceptedWord = "";
 
 	// stem filter
@@ -52,7 +53,7 @@ class ModeWords extends ModeCheonjiin {
 
 	@Override
 	protected void initPredictions() {
-		predictions = new WordPredictions(settings, textField);
+		predictions = new WordPredictions(settings);
 		predictions.setWordsChangedHandler(this::onPredictions);
 	}
 
@@ -87,14 +88,24 @@ class ModeWords extends ModeCheonjiin {
 
 
 	@Override
-	public boolean onNumber(int number, boolean hold, int repeat) {
+	public boolean onNumber(int number, boolean hold, int repeat, @NonNull String[] surroundingChars) {
 		isCursorDirectionForward = true;
-		return super.onNumber(number, hold, repeat);
+		return super.onNumber(number, hold, repeat, surroundingChars);
 	}
 
 
 	@Override
 	protected void onNumberPress(int nextNumber) {
+		// In East Asian languages, Spacebar must accept the current word, or type a space when there
+		// is no word. Here, we handle the case when 0-key is Space, and not the Space hotkey in
+		// HotkeyHandler, which could be a different key, assigned by the user.
+		if (ignoreNextSpace && nextNumber == Sequences.CHARS_0_KEY) {
+			ignoreNextSpace = false;
+			return;
+		}
+
+		ignoreNextSpace = false;
+
 		if (!language.isTranscribed() && containsGeneratedSuggestions() && digitSequence.length() - 1 == stem.length() && !predictions.getList().isEmpty() && predictions.getList().get(0).startsWith(stem)) {
 			// For alphabet-based languages, in case we are currently showing:
 			// > word + ... | ... a | ... b | ... c |
@@ -114,6 +125,7 @@ class ModeWords extends ModeCheonjiin {
 	@Override
 	protected void onNumberHold(int number) {
 		autoAcceptTimeout = 0;
+		ignoreNextSpace = false;
 		suggestions.add(language.getKeyNumeral(number));
 	}
 
@@ -144,7 +156,7 @@ class ModeWords extends ModeCheonjiin {
 	@Override
 	public String recompose() {
 		isRecomposing = false;
-		if (!language.hasSpaceBetweenWords() || language.isTranscribed()) {
+		if (textField == null || !language.hasSpaceBetweenWords() || language.isTranscribed()) {
 			return null;
 		}
 
@@ -154,7 +166,7 @@ class ModeWords extends ModeCheonjiin {
 		}
 
 		boolean includeApostrophes = LanguageKind.isUkrainian(language) || LanguageKind.isHebrew(language);
-		String previousWord = textField.getTextBeforeCursor(language, 50).getPreviousWord(false, includeApostrophes);
+		String previousWord = textField.getTextBeforeCursor(language, 50).getPreviousWord(false, includeApostrophes, false);
 		if (previousWord.length() < 2 || previousWord.contains(" ")) {
 			Logger.d(LOG_TAG, "Not recomposing invalid word: '" + previousWord + "'");
 			textCase = settings.getTextCase();
@@ -312,6 +324,8 @@ class ModeWords extends ModeCheonjiin {
 			.setInputWord(currentWord.isEmpty() ? stem : currentWord)
 			.setIsStemFuzzy(isStemFuzzy)
 			.setStem(stem)
+			.setAfterCursor(afterCursor)
+			.setBeforeCursor(beforeCursor)
 			.setDigitSequence(digitSequence)
 			.setLanguage(language)
 			.load();
@@ -384,17 +398,21 @@ class ModeWords extends ModeCheonjiin {
 
 	@Override
 	protected String adjustSuggestionTextCase(String word, int newTextCase) {
-		return autoTextCase.adjustSuggestionTextCase(new Text(language, word), newTextCase);
+		return language.hasUpperCase() ? autoTextCase.adjustSuggestionTextCase(new Text(language, word), newTextCase) : word;
 	}
 
 	@Override
-	public void determineNextWordTextCase(int nextDigit) {
+	public void determineNextWordTextCase(@Nullable String beforeCursor, int nextDigit) {
+		if (nextDigit >= 0 && !suggestions.isEmpty()) {
+			return;
+		}
+
 		final String nextSequence = nextDigit >= 0 ? digitSequence + nextDigit : digitSequence;
-		textCase = autoTextCase.determineNextWordTextCase(language, textCase, textFieldTextCase, textField, nextSequence, null);
+		textCase = autoTextCase.determineNextWordTextCase(language, textCase, textFieldTextCase, textField, nextSequence, beforeCursor);
 	}
 
 	private void determineTextFieldTextCase() {
-		int fieldCase = inputType.determineTextCase();
+		final int fieldCase = inputType != null ? inputType.determineTextCase() : CASE_UNDEFINED;
 		textFieldTextCase = allowedTextCases.contains(fieldCase) ? fieldCase : CASE_UNDEFINED;
 	}
 
@@ -442,8 +460,9 @@ class ModeWords extends ModeCheonjiin {
 		autoTextCase.skipNext();
 	}
 
+
 	@Override
-	public boolean shouldReplaceLastLetter(int n, boolean h) {
+	protected boolean shouldReplaceLastLetter(int n) {
 		return false;
 	}
 
@@ -455,6 +474,10 @@ class ModeWords extends ModeCheonjiin {
 	 */
 	@Override
 	public boolean shouldAcceptPreviousSuggestion(String currentWord, int nextDigit, boolean hold) {
+		if (!language.hasSpaceBetweenWords() && nextDigit == Sequences.CHARS_0_KEY && !digitSequence.isEmpty() && !seq.isAnySpecialCharSequence(digitSequence)) {
+			ignoreNextSpace = true;
+		}
+
 		if (hold) {
 			return true;
 		}
@@ -516,6 +539,7 @@ class ModeWords extends ModeCheonjiin {
 			// emojis and punctuation breaks words, unless there are database matches ('s, qu', по-, etc...)
 			&& digitSequence.contains(seq.CHARS_1_SEQUENCE)
 			&& !digitSequence.equals(seq.CHARS_1_SEQUENCE)
+			&& !digitSequence.equals(seq.CHARS_GROUP_1_SEQUENCE)
 			&& !digitSequence.startsWith(seq.EMOJI_SEQUENCE);
 	}
 
