@@ -1,5 +1,7 @@
 package io.github.sspanak.tt9.preferences.screens.deleteWords;
 
+import androidx.annotation.Nullable;
+
 import io.github.sspanak.tt9.R;
 import io.github.sspanak.tt9.preferences.PreferencesActivity;
 import io.github.sspanak.tt9.preferences.screens.BaseScreenFragment;
@@ -7,8 +9,8 @@ import io.github.sspanak.tt9.preferences.screens.BaseScreenFragment;
 public class DeleteWordsScreen extends BaseScreenFragment {
 	final public static String NAME = "DeleteWords";
 
-	public DeleteWordsScreen() { init(); }
-	public DeleteWordsScreen(PreferencesActivity activity) { init(activity); }
+	public DeleteWordsScreen() { super(); }
+	public DeleteWordsScreen(@Nullable PreferencesActivity activity) { super(activity); }
 
 	@Override public String getName() { return NAME; }
 	@Override protected int getTitle() { return R.string.pref_category_delete_words; }
@@ -19,6 +21,10 @@ public class DeleteWordsScreen extends BaseScreenFragment {
 	}
 
 	private void createPage() {
+		if (activity == null) {
+			return;
+		}
+
 		DeletableWordsList searchResultsList = new DeletableWordsList(activity.getSettings(), findPreference(DeletableWordsList.NAME));
 		searchResultsList.setTotalWords(0);
 		searchResultsList.setResult("", null);
